STYLESHEETS.append(data('cubes.conference.css'))
NOATTEND_ICON = data('noattend.jpeg')
ATTEND_ICON = data('attend.jpeg')

# Talk ressources
workflowColorAction = '#e5f5ee'
workflowColorWait = '#f5e5ee'



from datetime import date

from logilab.mtconverter import xml_escape
from logilab.common.date import date_range, ONEDAY

from cubicweb.predicates import is_instance, rql_condition, match_user_groups
from cubicweb.view import EntityView
from cubicweb.web.views import tabs

_ = unicode

class ConferencePrimaryView(tabs.TabbedPrimaryView):
    __select__ = is_instance('Conference')
    tabs = [_('confinfo'), _('talkslist'), _('talksschedule'),
            _('userconfschedule'), _('adminview')]
    default_tab = 'confinfo'

    def render_entity_title(self, entity):
        self.w(u'<h1>%s</h1>' % xml_escape(entity.dc_long_title()))


class ConferenceInfoView(EntityView):
    __select__ = is_instance('Conference')
    __regid__ = 'confinfo'

    def cell_call(self, row, col):
        entity = self.cw_rset.get_entity(row, col)
        _ = self._cw._
        if entity.description:
            self.w(u'<div>%s</div>' % entity.dc_description('text/html'))
        rql = ('Any TR, TD, MIN(ST), MAX(ET), COUNT(T) GROUPBY TR, '
              'TD WHERE T? in_track TR, T start_time ST, T end_time ET, '
              'TR description TD, TR in_conf C, C eid %(c)s')
        rset = self._cw.execute(rql, {'c':entity.eid})
        if rset:
            if entity.end_on > date.today():
                text = _(u'The conference will be divided into the following tracks:')
            else:
                text = _(u'The conference was divided into the following tracks:')
            self.w(u'<div>%s</div>' % text)
            headers = (_(u'Track title'), _(u'Description'), _(u'First talk'),
                       _(u'Last talk'), _(u'Total talks'))
            self.wview('table', rset, headers=headers, displaycols=range(0,5))
        else:
            self.w(u'<div>%s</div>' % _(u'No track yet in this conference.'))

class UserConfSchedule(EntityView):
    __select__ = is_instance('Conference') & ~match_user_groups('guests')
    __regid__ = 'userconfschedule'

    def cell_call(self, row, col):
        entity = self.cw_rset.get_entity(row, col)
        _ = self._cw._
        rql = ('Any T, ST, L ORDERBY ST WHERE T is Talk, T start_time ST, '
               'T location L, T in_conf C, C eid %(eid)s, U attend T, '
               'U login %(login)s')
        rset = self._cw.execute(rql, {'eid': entity.eid,
                                      'login': self._cw.user.login})
        if rset:
            self.w(u'<p>%s</p>' % _(u'You are planning to attend these talks:'))
            self.wview('table', rset, headers=(_(u'Talk title'),
                                               _(u'Date'), _(u'Location')))
        else:
            self.w(u'<p>%s</p>' % _(
                    u'Do you plan to attend a specific talk ? Please click on '
                    u'the attendancy button positionned at the top '
                    u'right of each talk summary.'))

class ConferenceTalksList(EntityView):
    __select__ = is_instance('Conference')
    __regid__ = 'talkslist'

    def cell_call(self, row, col):
        entity = self.cw_rset.get_entity(row, col)
        _ = self._cw._
        rql = ('Any T, group_concat(U), ST, L, TR GROUPBY T, ST, L, TR '
               'ORDERBY ST WHERE T location L, T start_time ST, '
               'U leads T, T in_conf C, T in_track TR, T in_state S, T title TT, '
               'S name "accepted", C eid %(c)s')
        rset = self._cw.execute(rql, {'c': entity.eid})
        if rset:
            headers = (_(u'Talk title'), _(u'Speaker'), _(u'Date'),
                       _(u'Location'), _(u'Track'))
            self.wview('table', rset, headers=headers, cellvids={1:'users_list'})
        else:
            self.w(u'<div>%s</div>' % _(u'No accepted talk yet in this conference'))

class ConferenceTalksSchedule(EntityView):
    __select__ = is_instance('Conference') & rql_condition('T in_conf X')
    __regid__ = 'talksschedule'

    def cell_call(self, row, col):
        entity = self.cw_rset.get_entity(row, col)
        tracks_rql = 'Any TR WHERE TR in_conf C, TR is Track, C eid %(c)s'
        tracks = list(self._cw.execute(tracks_rql, {'c': entity.eid}).entities())
        for day in date_range(entity.start_on, entity.end_on+ONEDAY):
            self.w(u'<table style="width:100%">')
            self.w(u'<tr>')
            for track in tracks:
                if entity.has_talk(day, track):
                    rset = self._cw.execute('Any T WHERE T in_track TR, TR eid %(e)s, '
                                            'T in_state S, S name "accepted"', {'e': track.eid})
                    if rset:
                        self.w(u'<td>')
                        self.wview('onedaycal', rset, initargs={'day':day, 'title':track.title})
                        self.w(u'</td>')
            self.w(u'</tr></table>')

class ConferenceAdminView(EntityView):
    __select__ = is_instance('Conference') & (match_user_groups('managers')
                                              | rql_condition('U is_chair_at X'))
    __regid__ = 'adminview'

    def cell_call(self, row, col):
        entity = self.cw_rset.get_entity(0,0)
        # number of talks in each state
        self.w(u'<h2>%s</h2>' % self._cw._(u'Talk state details'))
        entity = self.cw_rset.get_entity(0,0)
        rql = 'Any S, COUNT(T) GROUPBY S WHERE T is Talk, T in_state S, T in_conf C, C eid %(eid)s'
        rset = self._cw.execute(rql, {'eid': entity.eid})
        self.wview('table', rset, 'null')
        rset = self._cw.execute('Any T WHERE T is Talk, T in_state S, S name "submitted"')
        if rset:
            treid = self._cw.execute('Any T WHERE T is Transition, T name "send to reviewer"')[0][0]
            self.w(u'<div id="statreview">')
            self.w(u'<p>%s</p>' % self._cw._(u'Please, assign a reviewer to :'))
            self.w(u'<ul>')
            for talk in rset.entities():
                self.w('<li><a href="%s">%s</a></li>' %
                       (xml_escape(talk.absolute_url(vid='statuschange', treid=treid)),
                        xml_escape(talk.title)))
            self.w(u'</ul></div>')
        # number of participant by talk
        self.w(u'<h2>%s</h2>' % self._cw._(u'Number of participant by Talk'))
        rql = 'Any T, COUNT(A) GROUPBY T WHERE T is Talk, A attend T, T in_conf C, C eid %(eid)s'
        rset = self._cw.execute(rql, {'eid': entity.eid})
        self.wview('table', rset, 'null')




from logilab.mtconverter import xml_escape

from cubicweb.view import View
from cubicweb.predicates import is_instance, match_user_groups
from cubicweb.web.views import primary, uicfg
from cubes.conference.views.forms import (subject_in_track_vocabulary,
                                          subject_call_open_conf_vocabulary)

from cubicweb.web.views.basetemplates import HTMLPageHeader
HTMLPageHeader.headers = (('header-left', 'header-left'),
                          ('headtext', 'header-center'),
                          ('header-right', 'header-right'),
                         )

from cubicweb.web.views.basecomponents import ApplicationName
from cubicweb.web.views.ibreadcrumbs import BreadCrumbEntityVComponent
ApplicationName.context = BreadCrumbEntityVComponent.context = 'header-center'

_pvs = uicfg.primaryview_section
_pvs.tag_subject_of(('Talk', 'has_attachments', 'File'), 'hidden')
_pvs.tag_object_of(('CWUser', 'leads', 'Talk'), 'hidden')
_pvs.tag_object_of(('*', 'in_conf', '*'), 'hidden')
_pvs.tag_subject_of(('*', 'in_conf', '*'), 'hidden')
_pvs.tag_subject_of(('Talk', 'in_track', '*'), 'hidden')
_pvs.tag_object_of(('Talk', 'in_track', '*'), 'hidden')
_pvs.tag_subject_of(('CWUser', 'attend', '*'), 'hidden')
_pvs.tag_object_of(('CWUser', 'reviews', 'Talk'), 'hidden')
_pvs.tag_object_of(('*', 'attend', 'Talk'), 'hidden')
_pvs.tag_object_of(('Conference', 'take_place_at', 'PostalAddress'), 'relations')

_abaa = uicfg.actionbox_appearsin_addmenu
_abaa.tag_subject_of(('Talk', 'has_attachments', 'File'), True)
_abaa.tag_object_of(('Track', 'in_conf', 'Conference'), True)
_abaa.tag_object_of(('Talk', 'in_conf', 'Conference'), False)
_abaa.tag_object_of(('Talk', 'in_track', 'Track'), False)

_afs = uicfg.autoform_section
_afs.tag_object_of(('CWUser', 'leads', 'Talk'), formtype='main', section='hidden')
_afs.tag_subject_of(('Sponsor', 'has_logo', '*'), formtype='main', section='inlined')
_afs.tag_subject_of(('Talk', 'in_track', '*'), formtype='main', section='attributes')
_afs.tag_subject_of(('Conference', 'take_place_at', 'PostalAddress'), formtype="main", section="inlined")
_afs.tag_attribute(('PostalAddress', 'latitude'), formtype="main", section="attributes")
_afs.tag_attribute(('PostalAddress', 'longitude'), formtype="main", section="attributes")

uicfg.autoform_field_kwargs.tag_subject_of(('Talk', 'in_track', 'Track'),
                                           {'choices': subject_in_track_vocabulary})
uicfg.autoform_field_kwargs.tag_subject_of(('Talk', 'in_conf', 'Conference'),
                                           {'choices': subject_call_open_conf_vocabulary})
uicfg.autoform_field_kwargs.tag_subject_of(('Talk', 'has_attachments', 'File'),
                                           {'order': -1})

class UsersList(View):
    __regid__ = 'users_list'

    def cell_call(self, row, col):
        user_eids = self.cw_rset[row][col].split(',')
        users = []
        for user_eid in user_eids:
            rset = self._cw.execute('Any U WHERE U eid %(e)s', {'e': user_eid})
            users.append(rset.get_entity(0,0).dc_long_title())
        self.w(', '.join(users))


class CWUserPrimaryView(primary.PrimaryView):
    __select__ = is_instance('CWUser') & match_user_groups('guests')

    def render_entity_title(self, entity):
        title = xml_escape(entity.dc_long_title())
        self.w(u'<h1>%s</h1>' % title)

    def render_entity_attributes(self, entity):
        if entity.representing:
            self.w("%s %s" % (self._cw._(u'Representing'), entity.representing))

from cubicweb.web.views.urlrewrite import SimpleReqRewriter, rgx

class ConferenceTabRewrite(SimpleReqRewriter):
    rules = [
        (rgx('/conference/([^/]+)/([A-Za-z]+)'),
         dict(rql='Any C WHERE C is Conference, C url_id "%(id)s"' % {'id': r'\1'},
              selected_tab=r'\2', tab=r'\2')),
         ]

from cubes.seo.views import SitemapRule

class ConferenceSitemapRule(SitemapRule):
    __regid__ = 'conference'
    query = 'Any X WHERE X is Conference'
    priority = 1.0
    chfreq = 'monthly'

class TrackSitemapRule(SitemapRule):
    __regid__ = 'track'
    query = 'Any X WHERE X is Track'
    priority = 1.0
    chfreq = 'monthly'

class TalkSitemapRule(SitemapRule):
    __regid__ = 'talk'
    query = 'Any X WHERE X is Talk'
    priority = 1.0
    chfreq = 'monthly'




from cubicweb.web.action import Action, LinkToEntityAction
from cubicweb.web.views import actions
from cubicweb.predicates import (
    one_line_rset, has_permission, is_instance, match_user_groups,
    rql_condition, match_search_state, partial_relation_possible)

# don't add action to add a talk in uicfg
# (actionbox_appearsin_addmenu) because these actions must be display
# only for managers, users have a specific button (confCallForPaper)
# to add a talk
class AddTalkInConferenceAction(LinkToEntityAction):
    __regid__ = 'addtalkinconf'
    __select__ = (match_search_state('normal') & one_line_rset()
                  & partial_relation_possible(action='add', strict=False)
                  & is_instance('Conference')
                  & (match_user_groups('managers') | rql_condition('U is_chair_at X')))

    title = _('add Talk in_conf Conference object')
    target_etype = 'Talk'
    rtype = 'in_conf'
    role = 'object'

class AddTalkInTrackAction(LinkToEntityAction):
    __regid__ = 'addtalkintrack'
    __select__ = (match_search_state('normal') & one_line_rset()
                  & partial_relation_possible(action='add', strict=False)
                  & is_instance('Track')
                  & (match_user_groups('managers')
                     | rql_condition('U is_chair_at C, X is Track, X in_conf C'))
                 )

    title = _('add Talk in_track Track object')
    target_etype = 'Talk'
    rtype = 'in_track'
    role = 'object'

class ModifyAction(actions.ModifyAction):
    # remove has_permission('update') from __select__
    # indeed user can set "attend" relation to a talk
    # but this action is done with a specific button
    # in the talk primary view
    # we want to see the modify action only if user can
    # update the entity
    __select__ = (Action.__select__ & one_line_rset() &
                  has_permission('update'))

def registration_callback(vreg):
    vreg.unregister(actions.CopyAction)
    vreg.register_and_replace(ModifyAction, actions.ModifyAction)
    for entity in (AddTalkInConferenceAction, AddTalkInTrackAction):
        vreg.register(entity)



from cubicweb.web import formfields as ff, formwidgets as fwdgs
from cubicweb.web.views.workflow import (WFHistoryVComponent, ChangeStateFormView)
from cubicweb.predicates import (match_user_groups, rql_condition, is_instance)
from cubes.conference.views.forms import reviewers_vocabulary

class ConferenceWFHistoryVComponent(WFHistoryVComponent):
    __select__ = (WFHistoryVComponent.__select__ & is_instance('Talk') &
                  (match_user_groups('owners', 'managers')
                   | rql_condition('U reviews X')
                   | rql_condition('U is_chair_at C, X is Talk, X in_conf C')))


class SendToReviewerStatusChangeView(ChangeStateFormView):
    __select__ = (ChangeStateFormView.__select__ &
                  is_instance('Talk') & match_user_groups('managers') &
                  rql_condition('X in_state S, S name "submitted"'))

    def get_form(self, entity, transition, **kwargs):
        # XXX used to specify both rset/row/col and entity in case implements
        # selector (and not implements) is used on custom form
        form = self._cw.vreg['forms'].select(
            'changestate', self._cw, entity=entity, transition=transition,
            redirect_path=self.redirectpath(entity), **kwargs)

        relation = ff.RelationField(name='reviews', role='object',
                                    eidparam=True,
                                    label=_('select reviewers'),
                                    widget=fwdgs.Select(multiple=True),
                                    choices=reviewers_vocabulary)
        form.append_field(relation)
        trinfo = self._cw.vreg['etypes'].etype_class('TrInfo')(self._cw)
        trinfo.eid = self._cw.varmaker.next()
        subform = self._cw.vreg['forms'].select('edition', self._cw, entity=trinfo,
                                                mainform=False)
        subform.field_by_name('wf_info_for', 'subject').value = entity.eid
        trfield = subform.field_by_name('by_transition', 'subject')
        trfield.widget = fwdgs.HiddenInput()
        trfield.value = transition.eid
        form.add_subform(subform)
        return form

def registration_callback(vreg):
    vreg.register(SendToReviewerStatusChangeView)
    vreg.register_and_replace(ConferenceWFHistoryVComponent, WFHistoryVComponent)



from logilab.mtconverter import xml_escape

from cubicweb.view import EntityView
from cubicweb.predicates import is_instance, one_line_rset

class SwcConferenceView(EntityView):
    __regid__ = 'swc'
    __select__ = is_instance('Conference') & one_line_rset()
    title = _('swc')
    templatable = False
    content_type = 'application/rdf+xml'

    def call(self):
        self.w(u'''<?xml version="1.0" encoding="%s"?>
        <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
        xmlns:rdfs="http://www.w3org/2000/01/rdf-schema#"
        xmlns:swrc="http://swrc.ontoware.org/ontology#"
        xmlns:swc="http://data.semanticweb.org/ns/swc/ontology#"
        xmlns:foaf="http://xmlns.com/foaf/0.1/"
        xmlns:dc="http://purl.org/dc/elements/1.1/#"
        xmlns:ical="http://www.w3.org/2002/12/cal/ical#">\n''' % self._cw.encoding)
        self.cell_call(0, 0)
        self.w(u'</rdf:RDF>\n')

    def cell_call(self, row, col):
        entity = self.cw_rset.complete_entity(row, col)
        self.w(u'<swc:ConferenceEvent rdf:about="%s">\n' % entity.absolute_url())
        self.w(u'<dc:title>%s</dc:title>\n' % xml_escape(entity.dc_title()))
        self.w(u'<rdfs:label>%s</rdfs:label>' % xml_escape(entity.dc_title()))
        self.w(u'<swc:hasAcronym>%s</swc:hasAcronym>' % xml_escape(entity.url_id))
        self.w(u'<ical:dtstart rdf:parseType="Resource">'
               u'<ical:date>%s</ical:date></ical:dtstart>' % entity.start_on)
        self.w(u'<ical:dtend rdf:parseType="Resource">'
               u'<ical:date>%s</ical:date></ical:dtend>' % entity.end_on)
        self.w(u'<ical:url rdf:resource="%s"/>' % entity.absolute_url())
        tracks = self._cw.execute('Any T WHERE T is Track, T in_conf C, C eid %(x)s',
                                  {'x': entity.eid}).entities()
        for track in tracks:
            self.w(u'<swrc:isSuperEventOf rdf:resource="%s"/>\n' % track.absolute_url())
        self.w(u'</swc:ConferenceEvent>\n')

class SwcTrackView(EntityView):
    __regid__ = 'swc'
    __select__ = is_instance('Track') & one_line_rset()
    title = _('swc')
    templatable = False
    content_type = 'application/rdf+xml'

    def call(self):
        self.w(u'''<?xml version="1.0" encoding="%s"?>
        <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
        xmlns:rdfs="http://www.w3org/2000/01/rdf-schema#"
        xmlns:swrc="http://swrc.ontoware.org/ontology#"
        xmlns:swc="http://data.semanticweb.org/ns/swc/ontology#"
        xmlns:foaf="http://xmlns.com/foaf/0.1/"
        xmlns:dc="http://purl.org/dc/elements/1.1/#">\n''' % self._cw.encoding)
        self.cell_call(0, 0)
        self.w(u'</rdf:RDF>\n')

    def cell_call(self, row, col):
        entity = self.cw_rset.complete_entity(row, col)
        self.w(u'<swc:TrackEvent rdf:about="%s">' % entity.absolute_url())
        self.w(u'<dc:title>%s</dc:title>' % xml_escape(entity.dc_title()))
        for conf in entity.in_conf:
            self.w(u'<swc:isPartOf rdf:resource="%s"/>' % conf.absolute_url())
        for track in entity.reverse_in_track:
            self.w(u'<swrc:isSuperEventOf rdf:resource="%s"/>' % track.absolute_url())
        self.w(u'</swc:TrackEvent>')

class SwcTalkView(EntityView):
    __regid__ = 'swc'
    __select__ = is_instance('Talk')
    title = _('swc')
    templatable = False
    content_type = 'application/rdf+xml'

    def call(self):
        self.w(u'''<?xml version="1.0" encoding="%s"?>
        <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
        xmlns:rdfs="http://www.w3org/2000/01/rdf-schema#"
        xmlns:swrc="http://swrc.ontoware.org/ontology#"
        xmlns:swc="http://data.semanticweb.org/ns/swc/ontology#"
        xmlns:foaf="http://xmlns.com/foaf/0.1/"
        xmlns:dc="http://purl.org/dc/elements/1.1/#">\n''' % self._cw.encoding)
        for i in xrange(self.cw_rset.rowcount):
            self.cell_call(i, 0)
        self.w(u'</rdf:RDF>\n')

    def cell_call(self, row, col):
        entity = self.cw_rset.complete_entity(row, col)
        self.w(u'<swc:TalkEvent rdf:about="%s">' % entity.absolute_url())
        for conf in entity.in_conf:
            self.w(u'<swc:isPartOf rdf:resource="%s"/>' % conf.absolute_url())
        for track in entity.in_track:
            self.w(u'<swc:isPartOf rdf:resource="%s"/>' % track.absolute_url())
        self.w(u'<dc:title>%s</dc:title>' % xml_escape(entity.dc_title()))
        self.w(u'<swrc:abstract><![CDATA[%s]]></swrc:abstract>' % entity.description)
        for author in entity.reverse_leads:
            self.w(u'<swrc:author rdf:resource="%s"/>' % author.absolute_url())
        for doc in entity.has_attachments:
            self.w(u'<swc:hasRelatedDocument rdf:resource="%s"/>' % doc.absolute_url())
        self.w(u'</swc:TalkEvent>')




_ = unicode

from cubicweb.web import Redirect
from cubicweb.view import StartupView
from cubicweb.web.views import startup

from logilab.mtconverter import xml_escape

REPLACE = []

# index view
class ConferenceIndexView(startup.IndexView):
    __regid__ = 'index'
    title = _('Index')
    add_etype_links = ('Conference',)
    upcoming_conferences = ('Any C ORDERBY S DESC WHERE C is Conference, C start_on S, C end_on >= now')
    past_conferences = ('Any C ORDERBY S DESC WHERE C is Conference, C start_on S, C end_on < now')

    def call(self):
        rset = self._cw.execute('Any X WHERE X wikiid "indexpage"')
        if rset:
            entity = rset.get_entity(0,0)
            content = entity.printable_value('content', format='text/html')
            self.w(u'<div id="indexpage">')
            self.w(content)
            self.w(u'</div>')
        rset_next = self._cw.execute(self.upcoming_conferences)
        if rset_next:
            self.w(u'<h2>%s</h2>' % self._cw._(u'Upcoming conferences'))
            self.w(u'<ul>')
            for conf in rset_next.entities():
                self.w(u'<li>%s</li>' % conf.view('outofcontext'))
            self.w(u'</ul>')
        rset_past = self._cw.execute(self.past_conferences)
        if rset_past:
            self.w(u'<h2>%s</h2>' % self._cw._(u'Past conferences'))
            self.w(u'<ul>')
            for conf in rset_past.entities():
                self.w(u'<li>%s</li>' % conf.view('outofcontext'))
            self.w(u'</ul>')
        if not rset_next and not rset_past:
            self.w(self._cw._(u'No conference planned for now...'))

REPLACE.append( (ConferenceIndexView, startup.IndexView) )

# XXX move to cmt
class LoginOrRegister(StartupView):
    __regid__ = 'login_or_register'

    def call(self, title=True):
        redirpath = self._cw.form.get('__redirectpath')
        if self._cw.session.anonymous_session:
            if title:
                self.w(u'<h1>%s</h1>' % self._cw._(u'Authentication is needed to perform this action'))
            self.w(u'<h2>%s:</h2>' % self._cw._(u'If you already have an account, please log in'))
            form = self._cw.vreg['forms'].select('logform', self._cw, redirect_path=redirpath)
            form.render(w=self.w)
            self.w(u'<h2>%s:</h2>' % self._cw._(u'If you have an openid, please use it'))
            form = self._cw.vreg['forms'].select('openidlogform', self._cw)
            morebuttons = self._cw.vreg['buttons'].selectable('openid_button', self._cw)
            form.form_buttons = form.form_buttons[:]
            for button in morebuttons:
                form.form_buttons.insert(-1, button)
            form.render(w=self.w, display_progress_div=False)
            self.w(u'<h2>%s:</h2>' % self._cw._(u'If you do not have an account yet, please create one'))
            form = self._cw.vreg['forms'].select('registration', self._cw)
            form.render(w=self.w, display_progress_div=False)
        else:
            raise Redirect(self._cw.build_url(redirpath))
        # XXX would be even better to have a redirect message

def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__, [new for new,old in REPLACE])
    for new, old in REPLACE:
        vreg.register_and_replace(new, old)



from cubicweb.predicates import is_instance
from cubicweb.web.facet import RelationFacet, DateRangeFacet

# This selector is used by the TalkDate facet,
# the start_time date is optional.

class TalkDateFacet(DateRangeFacet):
    __regid__ = 'date-facet'
    __select__ = DateRangeFacet.__select__ & is_instance('Talk')
    rtype = 'start_time'

    @property
    def title(self):
        return self._cw._("Date")

class TalkTrackFacet(RelationFacet):
    __regid__ = 'track-facet'
    __select__ = RelationFacet.__select__ & is_instance('Talk')
    rtype = 'in_track'
    target_attr = 'title'

class TalkTagsFacet(RelationFacet):
    __regid__ = 'tags-facet'
    __select__ = RelationFacet.__select__ & is_instance('Talk')
    rtype = 'tags'
    target_attr = 'name'

class TalkConferenceFacet(RelationFacet):
    __regid__ = 'conference-facet'
    __select__ = RelationFacet.__select__ & is_instance('Talk')
    rtype = 'in_conf'
    target_attr = 'title'



from cubicweb.cwconfig import register_persistent_options

register_persistent_options((
    ('sponsor-contact-email',
     {'type' : 'string',
      'default': 'unset',
      'help': 'Sponsor contact email',
      'group': 'ui',
      'site_wide': True}),
     ))

from yams import xy

xy.register_prefix('swrc', 'http://swrc.ontoware.org/ontology')

xy.add_equivalence('Conference', 'swrc:ConferenceEvent')
xy.add_equivalence('CWUser login', 'foaf:name')
xy.add_equivalence('Talk', 'swrc:Paper')
xy.add_equivalence('Talk title', 'swrc:title')
xy.add_equivalence('Talk description', 'swrc:abstract')
xy.add_equivalence('Talk description', 'swrc:abstract')
xy.add_equivalence('Talk has_attachments', 'swrc:hasRelatedDocument')
xy.add_equivalence('Talk leads', 'swrc:author')



import datetime

from logilab.common.registry import objectify_predicate

from cubicweb.predicates import is_instance, has_related_entities
from cubicweb.server.hook import Hook, match_rtype, Operation, DataOperationMixIn
from cubicweb.sobjects.notification import (
    RecipientsFinder, NotificationView, StatusChangeMixIn)

from cubes.conference.utils import normalize_name


_ = unicode


def get_unique_url_id(execute, conference):
    url_id = normalize_name(conference.title)
    suffix = 0
    new_url_id = url_id
    while True:
        rset = execute('Any C WHERE C url_id %(url)s',
                       {'url': new_url_id})
        if not rset:
            return new_url_id
        suffix += 1
        new_url_id = '%s_%s' % (url_id, suffix)


class ConferenceUrlIdentifierHook(Hook):
    """
    If the conference has no url_id attribute, generate it automatically.
    """
    __regid__ = 'conference-url-identifier-hook'
    __select__ = Hook.__select__ & is_instance('Conference')
    events = ('before_add_entity',)

    def __call__(self):
        if not self.entity.url_id:
            url_id = get_unique_url_id(self._cw.execute, self.entity)
            self.entity.cw_edited['url_id'] = url_id

class AddTalkLeadHook(Hook):
    __regid__ = 'add-talk-lead-hook'
    __select__ = is_instance('Talk')
    events = ('after_add_entity',)

    def __call__(self):
        if not self._cw.is_internal_session:
            asession = self._cw
            asession.execute('SET U leads X WHERE X eid %(x)s, U eid %(u)s, NOT U attend X',
                             {'x': self.entity.eid, 'u': self._cw.user.eid})

class LeadsHook(Hook):
    """
    Create relation attend on CWUser who leads a Talk:
    U leads T => U attend T
    """
    __regid__ = 'lead-entails-attend-hook'
    __select__ = Hook.__select__ & match_rtype('leads')
    events = ('after_add_relation',)

    def __call__(self):
        if not self._cw.is_internal_session:
            asession = self._cw
            asession.execute('SET U attend X WHERE X eid %(x)s, U eid %(u)s, NOT U attend X',
                             {'x': self.eidto, 'u': self.eidfrom})


class SyncInConfOp(DataOperationMixIn, Operation):
    def precommit_event(self):
        for talk, track in self.get_data():
            self.cnx.execute('SET T in_conf C WHERE T eid %(t)s, T in_track K, K in_conf C, K eid %(k)s, NOT T in_conf C',
                             {'t': talk, 'k': track})

class InTrackHook(Hook):
    """
    When a Talk is added to a Track, automatically add it to the Conference:
    T in_track TR => T in_conf C
    """
    __regid__ = 'in_track-entails-in_conf-hook'
    __select__ = Hook.__select__ & match_rtype('in_track')
    events = ('before_add_relation',)

    def __call__(self):
        if not self._cw.is_internal_session:
            SyncInConfOp.get_instance(self._cw).add_data((self.eidfrom, self.eidto))


#-------------------
# Email notification
#-------------------

class TalkNotificationView(NotificationView):
    __select__ = NotificationView.__select__ & is_instance('Talk',)

    def context(self, **kwargs):
        context = super(TalkNotificationView, self).context(**kwargs)
        entity = self.cw_rset.complete_entity(0,0)
        context.update({'talk_name': entity.title,
                        'date': datetime.date.today().isoformat(),
                        'firstname': entity.reverse_leads[0].firstname,
                        'lastname': entity.reverse_leads[0].surname,
                        'conference': entity.in_conf[0].title})
        return context

    def recipients(self):
        recipients = []
        for finderid in self.recipients_finder:
            finder = self._cw.vreg['components'].select(finderid, self._cw,
                                                        rset= self.cw_rset)
            recipients += finder.recipients()
        return recipients

class TalkAddRecipientsFinder(RecipientsFinder):
    __select__ = RecipientsFinder.__select__ & is_instance('Talk',)
    __regid__ = 'manager_finder'

    @property
    def user_rql(self):
        return ('Any U, E, A WHERE U is CWUser, U in_group N, N name "managers", U primary_email E, E address A')

class TalkUpdateRecipientsFinder(RecipientsFinder):
    __select__ = RecipientsFinder.__select__ & is_instance('Talk',)
    __regid__ = 'interested_by_finder'

    @property
    def user_rql(self):
        return ('Any U, E, A WHERE X eid %(eid)s, EXISTS(U reviews X) '
                'OR EXISTS(U leads X), U primary_email E, E address A' %
                {'eid':  self.cw_rset[0][0]})

class TalkAdd(TalkNotificationView):
    """  Send an email to managers after a talk creation
    """
    __regid__ = 'notif_after_add_entity'

    recipients_finder = ('manager_finder',)

    content = _(u""
                u"Dear %(firstname)s %(lastname)s,\n"
                u"\n"
                u"The talk '%(talk_name)s' was added on %(date)s.\n"
                u"\n"
                u"Sincerely,\n"
                u"\n"
                u"--\n"
                u"The %(conference)s organizing committee.\n")

    def subject(self):
        entity = self.cw_rset.get_entity(0,0)
        return self._cw._('[%s] a talk was added') % entity.in_conf[0].title

# XXX move to framework ?
# XXX why metaattrsupdate is updated at before_update_entity ?
@objectify_predicate
def entity_really_edited(cls, req, rset=None, **kwargs):
    # when the status of an entity changes, the modification_date of the entity changes too
    # in case of talks, don't call talkupdate if only modification_date attribute has changed
    entity = rset.get_entity(0, 0)
    if getattr(entity, 'cw_edited', None) is None:
        return 0
    if len(entity.cw_edited) == 1 and 'modification_date' in entity.cw_edited:
        return 0
    return 1

class TalkUpdate(TalkNotificationView):
    __regid__ = 'notif_after_update_entity'
    recipients_finder = ('manager_finder', 'interested_by_finder',)
    __select__ = TalkNotificationView.__select__ & entity_really_edited()

    content = _(u""
                u"Dear %(firstname)s %(lastname)s,\n"
                u"\n"
                u"The talk '%(talk_name)s' was modified on %(date)s.\n"
                u"\n"
                u"Sincerely,\n"
                u"\n"
                u"--\n"
                u"The %(conference)s organizing committee.\n")

    def subject(self):
        entity = self.cw_rset.get_entity(0,0)
        return self._cw._('[%s] a talk was updated') % entity.in_conf[0].title

class TalkStatusUpdate(StatusChangeMixIn, TalkNotificationView):
    recipients_finder = ('manager_finder', 'interested_by_finder')

    content = _(u""
                u"Dear user,\n\n"
                u"The '%(title)s' status was updated from '%(previous_state)s' to '%(current_state)s'.\n\n"
                u"Sincerely,\n"
                u"\n"
                u"--\n"
                u"The conference organizing committee.\n")

    def subject(self):
        entity = self.cw_rset.get_entity(0,0)
        return _(u'[%s] talk status was updated' % entity.in_track[0].in_conf[0].title)



from yams.buildobjs import (
    EntityType, String, Date, SubjectRelation, Datetime, RichString,
    RelationDefinition, Boolean)
from yams.constraints import SizeConstraint, BoundaryConstraint, Attribute


from cubicweb.schema import (
    ERQLExpression, RRQLExpression, WorkflowableEntityType,
    RQLVocabularyConstraint)
from cubicweb.schemas.base import CWUser


_ = unicode

CWUser.add_relation(SubjectRelation('Talk', cardinality='**'), name='attend')

CWUser.add_relation(String(), name='representing')
CWUser.__permissions__['read'] = ('managers', 'users', 'guests')

def user_owns_subject():
    return RRQLExpression('S owned_by U')

def user_owns_object():
    return RRQLExpression('O owned_by U')

def user_is_chair():
    return ERQLExpression('U is_chair_at C, X in_conf C')

def user_is_chair_subject():
    return RRQLExpression('U is_chair_at C, S in_conf C')

def user_is_chair_object():
    return RRQLExpression('U is_chair_at C, O in_conf C')

def user_chairs_object():
    return RRQLExpression('U is_chair_at O')

class leads(RelationDefinition):
    subject = 'CWUser'
    object = 'Talk'
    cardinality = '*+'
    constraints = [RQLVocabularyConstraint('S in_group G, G name "users"')]
    __permissions__ = {
        'read':   ('managers', 'users', 'guests'),
        'add':    ('managers', user_is_chair_object(), user_owns_object()),
        'delete': ('managers', user_is_chair_object(), user_owns_object()),
        }

class Conference(WorkflowableEntityType):
    __permissions__ = {
        'read':   ('managers', 'users', 'guests',),
        'add':    ('managers', ),
        'update': ('managers', 'owners', ERQLExpression('U is_chair_at X'),),
        'delete': ('managers', 'owners',),
        }
    title = String(maxsize=50, required=True, fulltextindexed=True)
    url_id = String(maxsize=50, required=True, unique=True,
                    description=_('unique conference identifier for url'))
    start_on = Date(default='TODAY', required=True, constraints=[BoundaryConstraint('<=', Attribute('end_on'))])
    end_on = Date(default='TODAY', required=True, constraints=[BoundaryConstraint('>=', Attribute('start_on'))])
    description = RichString(fulltextindexed=True)
    call_open = Boolean(default=False)
    reg_open = Boolean(default=False)

class Track(EntityType):
    __permissions__ = {
        'read':   ('managers', 'users', 'guests'),
        'add':    ('managers', user_is_chair()),
        'update': ('managers', 'owners', user_is_chair()),
        'delete': ('managers', 'owners', user_is_chair()),
        }
    title = String(maxsize=50, required=True, fulltextindexed=True)
    description = RichString(fulltextindexed=True)

class Talk(WorkflowableEntityType):
    __permissions__ = {
        'read':   ('managers', user_is_chair(),
                   ERQLExpression('X owned_by U'),
                   ERQLExpression('X in_state ST, ST name "accepted"'),
                   ERQLExpression('U reviews X, X in_state ST, '
                                  'ST name in ("correction", "inreview", "rejected", '
                                  '"accept_pending", "reject_pending")'),
                   ),
        'add':    ('managers', 'users'),
        'update': ('managers', user_is_chair(),
                   ERQLExpression('X in_state ST, ST name in ("draft", "correction"), X owned_by U')),
        'delete': ('managers', user_is_chair()),
        }
    title = String(maxsize=200, required=True, fulltextindexed=True)
    description = RichString(fulltextindexed=True)
    start_time = Datetime(__permissions__={
        'read':   ('managers', 'users', 'guests',),
        'add':    ('managers', user_is_chair()),
        'update': ('managers', user_is_chair()),
        }, constraints=[BoundaryConstraint('<', Attribute('end_time'))])
    end_time = Datetime(__permissions__={
        'read':   ('managers', 'users', 'guests',),
        'add':    ('managers', user_is_chair()),
        'update': ('managers', user_is_chair()),
        }, constraints=[BoundaryConstraint('>', Attribute('start_time'))])

class comments(RelationDefinition):
    subject = 'Comment'
    object = 'Talk'
    cardinality='1*'

class location(RelationDefinition):
    __permissions__ = {
        'read':   ('managers', 'users', 'guests',),
        'add':    ('managers', user_is_chair()),
        'update': ('managers', user_is_chair())
        }
    subject = 'Talk'
    object = 'String'
    cardinality = '?1'
    constraints=[SizeConstraint(100)]

class in_track(RelationDefinition):
     __permissions__ = {
        'read':   ('managers', 'users', 'guests'),
        'add':    ('managers', user_is_chair_subject(), user_owns_subject()),
        'delete': ('managers', user_is_chair_subject(), user_owns_subject()),
        }
     subject = 'Talk'
     object = 'Track'
     cardinality = '?*'

class has_attachments(RelationDefinition):
    __permissions__ = {
        'read':   ('managers', 'users', 'guests'),
        'add':    ('managers', user_is_chair_subject(), user_owns_subject()),
        'delete': ('managers', user_is_chair_subject(), user_owns_subject()),
        }
    subject = 'Talk'
    object = 'File'
    cardinality = '*?'

class tags(RelationDefinition):
    __permissions__ = {
        'read':   ('managers', 'users', 'guests'),
        'add':    ('managers', user_is_chair_object()),
        'delete': ('managers', user_is_chair_object()),
        }
    subject = 'Tag'
    object = 'Talk'

class track_in_conf(RelationDefinition):
    __permissions__ = {
        'read':   ('managers', 'users', 'guests',),
        'add':    ('managers', user_chairs_object()),
        'delete': ('managers', user_chairs_object()),
        }
    name = 'in_conf'
    subject = 'Track'
    object = 'Conference'
    cardinality = '1*'

class talk_in_conf(RelationDefinition):
    __permissions__ = {
        'read':   ('managers', 'users', 'guests',),
        'add':    ('managers', 'users'),
        'delete': ('managers', user_chairs_object()),
        }
    name = 'in_conf'
    subject = 'Talk'
    object = 'Conference'
    cardinality = '1*'

class reviews(RelationDefinition):
    __permissions__ = {
        'read':   ('managers', 'users',),
        'add':    ('managers', user_chairs_object()),
        'delete': ('managers', user_chairs_object()),
        }
    subject = ('CWUser',)
    object = 'Talk'
    cardinality = '**'

class Sponsor(EntityType):
    __permissions__ = {
        'read':   ('managers', 'users', 'guests',),
        'add':    ('managers', user_is_chair()),
        'update': ('managers', 'owners', user_is_chair()),
        'delete': ('managers', 'owners'),
        }
    title = String(maxsize=100, required=True, fulltextindexed=True)
    description = RichString(fulltextindexed=True)
    url = String(maxsize=100)

class has_logo(RelationDefinition):
    __permissions__ = {
        'read':   ('managers', 'users', 'guests',),
        'add':    ('managers', user_owns_subject()),
        'delete': ('managers', user_owns_subject()),
        }
    subject = ('Sponsor',)
    object = 'File'
    cardinality = '1*'
    composite = 'subject'

# Sponsor is_sponsor SponsorShip sponsoring_conf Conference

class SponsorShip(EntityType):
    __permissions__ = {
        'read':   ('managers', 'users', 'guests',),
        'add':    ('managers', user_is_chair()),
        'update': ('managers', user_is_chair()),
        'delete': ('managers', user_is_chair()),
        }
    title = String(maxsize=100, required=True, fulltextindexed=True)
    level = String(vocabulary=('Gold', 'Silver', 'Bronze', 'Media'))

class sponsoring_conf(RelationDefinition):
    __permissions__ = {
        'read':   ('managers', 'users', 'guests',),
        'add':    ('managers', user_chairs_object()),
        'delete': ('managers', user_chairs_object()),
        }
    subject = ('SponsorShip',)
    object = 'Conference'
    cardinality = '+*'

class is_sponsor(RelationDefinition):
    __permissions__ = {
        'read':   ('managers', 'users', 'guests',),
        'add':    ('managers', ),
        'delete': ('managers', ),
        }
    subject = ('Sponsor',)
    object = ('SponsorShip',)
    cardinality = '**'

# roles

class is_chair_at(RelationDefinition):
    __permissions__ = {
        'read':   ('managers', 'users', 'guests',),
        'add':    ('managers', ),
        'delete': ('managers', ),
        }
    subject = ('CWUser',)
    object = ('Conference',)
    cardinality = '**'

class is_reviewer_at(RelationDefinition):
    __permissions__ = {
        'read':   ('managers', 'users', 'guests',),
        'add':    ('managers', user_chairs_object()),
        'delete': ('managers', user_chairs_object()),
        }
    subject = ('CWUser',)
    object = ('Conference',)
    cardinality = '**'

class take_place_at(RelationDefinition):
    __permissions__ = {
        'read':   ('managers', 'users', 'guests',),
        'add':    ('managers', user_is_chair_subject()),
        'delete': ('managers', user_is_chair_subject())
        }
    name = 'take_place_at'
    subject = 'Conference'
    object = ('PostalAddress',)
    cardinality = '+*'



"""this contains the template-specific entities' classes"""

__docformat__ = "restructuredtext en"

from logilab.common.date import todate

from cubicweb.predicates import is_instance
from cubicweb.entities import AnyEntity, adapters, fetch_config
from cubicweb.entities.authobjs import CWUser
from cubicweb.web.views.calendar import ICalendarableAdapter
from cubicweb.web.views.ibreadcrumbs import IBreadCrumbsAdapter
from cubicweb.view import EntityAdapter

# cwuser

class ConferenceCWUser(CWUser):

    def dc_long_title(self):
        name = super(ConferenceCWUser, self).name()
        if self.representing:
            return u'%s (%s)' % (name, self.representing)
        else:
            return name

# conference

class Conference(AnyEntity):
    __regid__ = 'Conference'

    rest_attr = 'url_id'

    def dc_long_title(self):
        if self.start_on.month == self.end_on.month:
            dates = u'%s %s-%s' % (self.start_on.strftime('%B'), self.start_on.day, self.end_on.day)
        else:
            dates = u'%s - %s' % (self.start_on.strftime('%B %d'), self.end_on.strftime('%B %d'))
        city = self.take_place_at[0].city
        return u'%s - %s %s' % (self.title, dates, city)

    def get_talks_by_track(self, track):
        rql = 'Any T WHERE T in_track TR, TR eid %(tr)s'
        return self._cw.execute(rql, {'tr': track.eid})

    def has_talk(self, day, track):
        for talk in self.get_talks_by_track(track).entities():
            if talk.is_on_day(day):
                return True
        return False

class ConferenceITreeAdapter(adapters.ITreeAdapter):
    __select__ = is_instance('Conference')

    tree_relation = 'in_conf'

class ConferenceIBreadCrumbsAdapter(IBreadCrumbsAdapter):
    __select__ = is_instance('Conference')

    def breadcrumbs(self, view=None, recurs=False):
        return [(self.entity.absolute_url(), self.entity.dc_title())]

# track

class Track(AnyEntity):
    __regid__ = 'Track'
    fetch_attrs, cw_fetch_order = fetch_config(['title'])

class TrackICalendarableAdapter(ICalendarableAdapter):
    __select__ = is_instance('Track')

    @property
    def start(self):
        rql_date = ("Any MIN(ST) WHERE T eid %(track)s, X in_track T,"
                    "X start_time ST, X in_state S, S name 'accepted'")
        rset = self._cw.execute(rql_date, {"track": self.entity.eid})
        # Aggregate function can return None (#1381429)
        if len(rset) and rset[0][0] is not None:
            return rset[0][0]

    @property
    def stop(self):
        rql_date = ("Any MAX(ET) WHERE T eid %(track)s, X in_track T,"
                    "X end_time ET, X in_state S, S name 'accepted'")
        rset = self._cw.execute(rql_date, {"track": self.entity.eid})
        # Aggregate function can return None (#1381429)
        if len(rset) and rset[0][0] is not None:
            return rset[0][0]

class TrackITreeAdapter(adapters.ITreeAdapter):
    __select__ = is_instance('Track')

    tree_relation = 'in_track'

class TrackIBreadCrumbsAdapter(IBreadCrumbsAdapter):
    __select__ = is_instance('Track')

    def breadcrumbs(self, view=None, recurs=False):
        breadcrumbs = []
        if self.entity.in_conf:
            breadcrumbs = self.entity.in_conf[0].cw_adapt_to('IBreadCrumbs').breadcrumbs(view, True)
        breadcrumbs.append(self.entity)
        return breadcrumbs

# talk

class Talk(AnyEntity):
    __regid__ = 'Talk'
    fetch_attrs, cw_fetch_order = fetch_config(['start_time', 'end_time'])

    def is_on_day(self, day):
        date = todate(day)
        if self.start_time:
            talk_date = todate(self.start_time)
            return date == talk_date
        else:
            return False

    @property
    def track(self):
        # track is not mandatory in schema by default
        if self.in_track:
            return self.in_track[0]


class TalkIBreadCrumbsAdapter(IBreadCrumbsAdapter):
    __select__ = is_instance('Talk')

    def breadcrumbs(self, view=None, recurs=False):
        try:
            breadcrumbs = self.entity.in_track[0].cw_adapt_to('IBreadCrumbs').breadcrumbs(view, True)
        except IndexError:
            breadcrumbs = []
        breadcrumbs.append(self.entity)
        return breadcrumbs

class TalkICalendarableAdapter(EntityAdapter):
    __regid__ = 'ICalendarable'
    __select__ = is_instance('Talk')

    @property
    def start(self):
        return self.entity.start_time

    @property
    def stop(self):
        return self.entity.end_time



# Sponsor migration

add_entity_type('SponsorShip', auto=True, commit=True)

rset = rql('Any S WHERE S is Sponsor')

sp_hash = {}
for sponsor in rset.entities():
    key = (sponsor.level, tuple(sponsor.supports_conf))
    sp_hash.setdefault(key, []).append(sponsor)

urql = session.unsafe_execute

for (level, confs), sponsors in sp_hash.items():
    sponsors_eid = ','.join(str(i.eid) for i in sp_hash[key])
    confs_eid = ','.join([str(i.eid) for i in confs])
    rset = urql("INSERT SponsorShip X: X level %(level)s, X title %(title)s",
               {'level': level, 'title': u'automatic title'})
    sponsorship = rset.get_entity(0,0)
    urql("SET X sponsoring_conf Y WHERE X eid %s, Y eid IN %s)" %
        (sponsorship.eid, confs_eid))
    urql("SET X is_sponsor Y WHERE X eid IN (%(sponsors_eid)s), Y eid %(sponsorship_eid)s" %
        {'sponsorship_eid': sponsorship.eid,
         'sponsors_eid': sponsors_eid})

drop_attribute('Sponsor', 'level', commit=True)
drop_relation_definition('Sponsor', 'supports_conf', 'Conference', commit=True)

commit()



# -*- coding: utf-8 -*-

from logilab.mtconverter import xml_escape
from cubicweb.web.views import boxes
from cubicweb.web.component import EntityCtxComponent, EmptyComponent
from cubicweb.web.htmlwidgets import BoxLink, BoxWidget
from cubicweb.predicates import is_instance, authenticated_user

# XXX move to cmt
class TalkStatusBox(boxes.BoxTemplate):
    __regid__ = 'talk-status'
    __select__ = boxes.BoxTemplate.__select__ & authenticated_user()
    context = 'left'
    title = _(u'your talks')
    order = 2

    def call(self, **kwargs):
        _cw = self._cw
        rset = _cw.execute('Any T WHERE T is Talk, U leads T, U eid %(x)s, '
                           'T in_conf C, C in_state S, S name "scheduled"',
                           {'x': _cw.user.eid})
        if not rset:
            return
        talks = {}
        for talk in rset.entities():
            talks.setdefault(talk.cw_adapt_to('IWorkflowable').state, []).append(talk)
        box = BoxWidget(_cw._(self.title), id=self.__regid__, islist=True)
        rql_ = ('Any T,C WHERE T is Talk, T in_conf C, U leads T, U eid %s, '
                'T in_state S, S name "%%s", C in_state CS, CS name "scheduled"' % _cw.user.eid)
        for state in 'draft submitted inreview correction accepted rejected'.split():
            msg = _cw._('Your talks in state %s') % _cw._(state)
            if state in talks:
                box.append(BoxLink(_cw.build_url('view', rql=rql_%state, vtitle=msg),
                                   xml_escape('%s %s' % (len(talks[state]), state))))
        box.render(w=self.w)

# XXX move to cmt
class ReviewStatusBox(boxes.BoxTemplate):
    __regid__ = 'review-status'
    __select__ = boxes.BoxTemplate.__select__ & authenticated_user()

    context = 'left'
    title = _(u'your reviews')
    order = 3

    def call(self, **kwargs):
        _cw = self._cw
        rset = self._cw.execute('Any T WHERE T is Talk, U reviews T, U eid %(x)s',
                                {'x': _cw.user.eid})
        if not rset:
            return
        talks = {}
        for talk in rset.entities():
            talks.setdefault(talk.cw_adapt_to('IWorkflowable').state, []).append(talk)
        box = BoxWidget(_cw._(self.title), id=self.__regid__, islist=True)
        rql_ = ('Any T,C WHERE T is Talk, T in_conf C, U reviews T, U eid %s, '
                'T in_state S, S name "%%s"' % _cw.user.eid)
        for state in ['inreview', 'correction', 'accept_pending', 'reject_pending']:
            msg = _cw._('Talks you have to review (in state %s)') % self._cw._(state)
            if state in talks:
                box.append(BoxLink(_cw.build_url('view', rql=rql_%state, vtitle=msg),
                                   xml_escape('%s %s' % (len(talks[state]), state))))
        if box.items:
            box.render(w=self.w)

# XXX move to cmt
class CallForPaperBox(boxes.BoxTemplate):
    __regid__ = 'callbox'

    context = 'left'
    title = _(u'Call for paper')
    order = 1

    def call(self, **kwargs):
        _cw = self._cw
        rset = _cw.execute('Any C WHERE C is Conference, C call_open True')
        if not rset:
            return
        w = self.w
        w(u'<div class="conferenceBox">')
        w(u'<div id="confCallForPaper">')
        if _cw.session.anonymous_session:
            msg = _cw._('You need to create an account and sign in '
                        'before you can submit an abstract')
            url = xml_escape(_cw.build_url('view', vid='login_or_register', __message=msg, __redirectpath='add/Talk'))
        elif _cw.user.leads:
            msg = _cw._(u'Remember you have already submitted a talk.')
            url = xml_escape(_cw.build_url('add/Talk', __message=msg))
        else:
            url = xml_escape(_cw.build_url('add/Talk'))
        w(u'<a href="%s">%s</a>' %
          (url, xml_escape(_cw._('Answer the call for talks !'))))
        w(u'</div></div>')


class SponsorBox(EntityCtxComponent):
    __regid__ = domid = 'sponsorbox'
    __select__ = EntityCtxComponent.__select__ & is_instance('Conference','Talk')
    context = 'right'
    title = 'Sponsors'

    def init_rendering(self):
        if self.entity.e_schema.type == 'Conference':
            rql = ('Any L, X, I WHERE I is File, X has_logo I, X is_sponsor S, S level L, S sponsoring_conf C, C eid %(conf)s')
            self.cw_rset = self._cw.execute(rql, {'conf': self.entity.eid})
        elif self.entity.e_schema.type == 'Talk':
            rql = ('Any L, X, I WHERE I is File, X has_logo I, X is_sponsor S, S level L, S sponsoring_conf C, T in_conf C, T eid %(talk)s')
            self.cw_rset = self._cw.execute(rql, {'talk': self.entity.eid})

    def render_body(self, w):
        # we can't use EmptyComponent in init_rendering cos we want to display
        # an informational message at the end
        _cw = self._cw
        sponsors = [(level, seid, imgeid) for (level, seid, imgeid) in self.cw_rset]
        if sponsors:
            # sort sponsor by level
            lhash = {u'Gold': 3, u'Silver': 2, u'Bronze': 1, u'Media': 1}
            sponsors.sort(key=lambda x: lhash[x[0]])
            sponsors.reverse()
            _level = None
            for (level, sponsor_eid, image_eid) in sponsors:
                if level != _level:
                    w(u'<div>%s</div>' % level)
                    _level = level
                sponsor = _cw.entity_from_eid(sponsor_eid)
                img = _cw.entity_from_eid(image_eid)
                url = img.cw_adapt_to("IDownloadable").download_url()
                # adapt image size
                w(u'<div class="%s">' % level)
                w(u'<a href="%s"><img src="%s" alt="%s"/></a>' %
                  (sponsor.url, url, xml_escape(img.dc_title())))
                w(u'</div>')
        w(u'<p>%s <a href="mailto:%s">%s</a></p>' % (_cw._('If you are interested in sponsoring or partnering'),
                                                     _cw.property_value('ui.sponsor-contact-email'),
                                                     _cw._('please contact us')))


class AtTheSameTimeBox(EntityCtxComponent):
    __regid__ = 'atthesametimebox'
    __select__ = EntityCtxComponent.__select__ & is_instance('Talk')
    context = 'incontext'
    title = _(u'At the same time')

    def init_rendering(self):
        self.at_the_same_time = []
        rql   = ('Any T WHERE T is Talk, NOT T eid %(eid)s, NOT T start_time NULL AND NOT T end_time NULL')
        rset  = self._cw.execute(rql, {'eid': self.entity.eid})
        start, end = self.entity.start_time, self.entity.end_time
        if rset and start is not None and end is not None:
            for talk in rset.entities():
                s = talk.start_time
                e = talk.end_time
                if (start <= s < end) or (start < e <= end):
                    self.at_the_same_time.append(talk)
        if not self.at_the_same_time:
            raise EmptyComponent()

    def render_body(self, w):
        for talk in self.at_the_same_time:
            w(u'<p><a href="%s">%s</a></p>' % (talk.absolute_url(), xml_escape(talk.dc_title())))



from cubicweb.web import component
from cubicweb.predicates import is_instance, match_user_groups, rql_condition

class TalkStateCtxComponent(component.EntityCtxComponent):
    __regid__ = 'talkstate'
    __select__ = (component.EntityCtxComponent.__select__
                  & is_instance('Talk') & match_user_groups('owners'))
    context = 'navcontenttop'

    def init_rendering(self):
        _cw = self._cw
        state = self.entity.cw_adapt_to('IWorkflowable').state
        color_action = _cw.uiprops.get('workflowColorAction')
        color_wait = _cw.uiprops.get('workflowColorWait')
        if state in ('draft', 'correction'):
            self.msg = (_cw._(u'This talk is in state "%s", which means it needs to be '
                              'submitted before it can be considered by the reviewers.')
                        % _cw._(state))
            self.color = color_action
        elif state in ('submitted', 'inreview'):
            self.msg = (_cw._(u'This talk is in state "%s", which means the reviewers '
                              'will soon send you their comments.')
                        % _cw._(state))
            self.color = color_wait
        elif state in ('accept_pending', 'reject_pending'):
            self.msg = _cw._(u'This talk was reviewed and is now awaiting a final '
                             'decision by the conference chairs.')
            self.color = color_wait
        else:
            raise component.EmptyComponent()

    def render_body(self, w):
        w(u'<div style="background-color: %s">' % self.color)
        w(self.msg)
        w(u'</div>')

class TalkStateCtxComponent2(TalkStateCtxComponent):
    __regid__ = 'talkstate2'
    __select__ = (component.EntityCtxComponent.__select__
                  & is_instance('Talk') & rql_condition('U reviews X'))

    def init_rendering(self):
        _cw = self._cw
        state = self.entity.cw_adapt_to('IWorkflowable').state
        color_action = _cw.uiprops.get('workflowColorAction')
        color_wait = _cw.uiprops.get('workflowColorWait')
        if state == 'inreview':
            self.msg = (_cw._(u'This talk is in state "%s", which means you have '
                              'to review it. You can ask the author for modifications, '
                              'accept or reject this talk.')
                        % _cw._(state))
            self.color = color_action
        elif state == 'correction':
            self.msg = (_cw._(u'This talk is in state "%s", which means the author '
                              'needs to answer the comments of the reviewers.')
                        % _cw._(state))
            self.color = color_wait
        elif state in ('accept_pending', 'reject_pending'):
            self.msg = _cw._(u'This talk was reviewed and is now awaiting a final '
                             'decision by the conference chairs.')
            self.color = color_wait
        else:
            raise component.EmptyComponent()




"""template automatic tests"""

from logilab.common.testlib import TestCase, unittest_main

from cubicweb.devtools.testlib import AutomaticWebTest
from cubicweb.devtools.fill import ValueGenerator

class MyValueGenerator(ValueGenerator):
    def generate_Sponsor_url(self, entity, index):
        return u'http://sponsor.example.com'

# necessary trick to avoid cw assertion error
class AutomaticWebTest(AutomaticWebTest): pass

if __name__ == '__main__':
    unittest_main()



CHAIR = 'U is_chair_at C, X in_conf C'
REVIEWER = 'U is_reviewer_at C, X in_conf C'
MANAGER = ('managers',)

twf = rql('Any W WHERE W is Workflow, W workflow_of X, X name "Talk"', ask_confirm=False).get_entity(0,0)
for state in twf.reverse_state_of:
    if state.name == 'accepted':
        accepted = state
    elif state.name == 'rejected':
        rejected = state
    elif state.name == 'inreview':
        inreview = state

# add new states
accept_pending = twf.add_state(_('accept_pending'))
reject_pending = twf.add_state(_('reject_pending'))

# modify existing transitions
for transition in twf.reverse_transition_of:
    if transition.name == u'accept talk':
        transition.set_attributes(name=u'propose to accept')
        transition.set_relations(destination_state=accept_pending)
    elif transition.name == u'reject talk':
        transition.set_attributes(name=u'propose to reject')
        transition.set_relations(destination_state=reject_pending)

# add new transitions
twf.add_transition(_('accept talk'), (accept_pending,reject_pending), accepted, MANAGER, conditions=CHAIR)
twf.add_transition(_('reject talk'), (accept_pending,reject_pending), rejected, MANAGER, conditions=CHAIR)
twf.add_transition(_('need more review'), (accept_pending,reject_pending), inreview, MANAGER, conditions=(REVIEWER,CHAIR))

commit()

sync_schema_props_perms()



from cubicweb.predicates import is_instance, match_user_groups
from cubicweb.web.views import autoform

# vocabulary for add / talk form

def reviewers_vocabulary(form, field):
    entity = form.edited_entity
    rset = form._cw.execute('Any U WHERE U is_reviewer_at C, X in_conf C, '
                            'X is Talk, X eid %(x)s', {'x': entity.eid})
    return [(item.dc_long_title(), item.eid) for item in rset.entities()]

def subject_call_open_conf_vocabulary(form, field):
    options = []
    edited_entity = form.edited_entity
    if edited_entity.has_eid() and edited_entity.in_conf:
        conf = edited_entity.in_conf[0]
        options = [(conf.dc_title(), conf.eid)]
    if form._cw.user.matching_groups('managers'):
        rset = form._cw.execute('Any C WHERE C is Conference')
    else:
        rset = form._cw.execute('Any C WHERE C is Conference, C call_open True, '
                                'C in_state S, S name "scheduled"')
    for entity in rset.entities():
        if edited_entity.has_eid() and conf.eid == entity.eid:
            continue
        options.append((entity.dc_title(), entity.eid))
    return options

def subject_reg_open_conf_vocabulary(form, field):
    options = []
    edited_entity = form.edited_entity
    if edited_entity.has_eid() and edited_entity.book_conf:
        conf = edited_entity.book_conf[0]
        options = [(conf.dc_title(), conf.eid)]
    if form._cw.user.matching_groups('managers'):
        rset = form._cw.execute('Any C WHERE C is Conference')
    else:
        rset = form._cw.execute('Any C WHERE C is Conference, C reg_open True, '
                                'C in_state S, S name "scheduled"')
    for entity in rset.entities():
        if edited_entity.has_eid() and conf.eid == entity.eid:
            continue
        options.append((entity.dc_title(), entity.eid))
    return options

def subject_in_track_vocabulary(form, field):
    options = []
    edited_entity = form.edited_entity
    if edited_entity.has_eid() and edited_entity.in_track:
        track = edited_entity.in_track[0]
        options = [(track.dc_title(), track.eid)]
    if form._cw.user.matching_groups('managers'):
        rset = form._cw.execute('Any T WHERE T is Track')
    else:
        rset = form._cw.execute('DISTINCT Any T WHERE T is Track, T in_conf C, '
                                'C call_open True, C in_state S, S name "scheduled"')
    for entity in rset.entities():
        if edited_entity.has_eid() and track.eid == entity.eid:
            continue
        options.append((entity.dc_title(), entity.eid))
    return options

class TalkAutoForm(autoform.AutomaticEntityForm):
    __select__ = autoform.AutomaticEntityForm.__select__ & is_instance('Talk') & match_user_groups('managers')

    def editable_relations(self):
        result = super(TalkAutoForm, self).editable_relations()
        rschema = self._cw.vreg.schema.rschema('leads')
        result.append( (rschema.display_name(self.edited_entity._cw, 'object', 'Talk'),
                        rschema, 'object') )
        return sorted(result)





# add attributes to Conference
add_attribute('Conference', 'call_open')
add_attribute('Conference', 'reg_open')

# add relations
add_relation_type('is_chair_at')
add_relation_type('is_reviewer_at')
add_relation_type('is_delegate_at')

# assuming the db is correct, propagate
rql('SET U is_reviewer_at C WHERE U reviews T, T in_conf C')

commit()

# remove group moderators
rql('SET X in_group U WHERE X in_group G, G name "moderators", U is CWGroup, U name "users"')
rql('DELETE CWGroup G WHERE G name "moderators"')

# get talk workflow
twf = rql('Any W WHERE W is Workflow, W workflow_of X, X name "Talk"', ask_confirm=False).get_entity(0,0)

# fix old migration problem
for transition in twf.reverse_transition_of:
    if transition.name == u'ask for correction':
        transition.set_attributes(name=u'need correction')

# add redraft transition
for state in twf.reverse_state_of:
    if state.name == 'draft':
        draft = state
    elif state.name == 'submitted':
        submitted = state
twf.add_transition(_('redraft'), (submitted,), draft, ('owners','managers',))

# change workflow permissions
CHAIR = 'U is_chair_at C, X in_conf C'
REVIEWER = 'U is_reviewer_at C, X in_conf C'
OWNER_CALL_OPEN = 'X owned_by U, X in_conf C, C call_open "True"'
MANAGER = ('managers',)
OWNER_MANAGER = ('owners', 'managers')

for transition in twf.reverse_transition_of:
    if transition.name == 'submit your work':
        transition.set_permissions(MANAGER, conditions=(CHAIR, OWNER_CALL_OPEN), reset=True)
    elif transition.name == 'redraft':
        transition.set_permissions(MANAGER, conditions=(CHAIR, OWNER_CALL_OPEN), reset=True)
    elif transition.name == 'send to reviewer':
        transition.set_permissions(MANAGER, conditions=CHAIR, reset=True)
    elif transition.name == 'need correction':
        transition.set_permissions(MANAGER, conditions=(REVIEWER,CHAIR), reset=True)
    elif transition.name == 'resend to reviewer':
        transition.set_permissions(OWNER_MANAGER, conditions=CHAIR, reset=True)
    elif transition.name == 'accept talk':
        transition.set_permissions(MANAGER, conditions=(REVIEWER,CHAIR), reset=True)
    elif transition.name == 'reject talk':
        transition.set_permissions(MANAGER, conditions=(REVIEWER,CHAIR), reset=True)
    else:
        raise Exception('unknown transition %s' % repr(transition.name))

commit()

sync_schema_props_perms()



# custom application views
_ = unicode

from datetime import date

from logilab.mtconverter import xml_escape
from logilab.common.date import date_range, todate, todatetime

from cubicweb.predicates import adaptable, empty_rset
from cubicweb.view import EntityView
from cubicweb.web.views import calendar

# one day calendar view for day of conference
class OneDayCal(EntityView):
    """Render a one day calendar view"""
    __regid__ = 'onedaycal'
    __select__ = empty_rset | adaptable('ICalendarable')
    paginable = False
    title = _('one day')

    def call(self):
        self._cw.add_js( ('cubicweb.ajax.js', 'cubicweb.calendar.js') )
        self._cw.add_css('cubicweb.calendar.css')
        task_colors = {}   # remember a color assigned to a task
        day = self.cw_extra_kwargs.get('day', date.today())
        title = self.cw_extra_kwargs.get('title', '')
        # colors here are class names defined in cubicweb.css
        colors = [ "col%x" % i for i in range(12) ]
        next_color_index = 0
        done_tasks = []
        task_descrs = []
        day = todate(day)
        for row in xrange(self.cw_rset.rowcount):
            task = self.cw_rset.get_entity(row, 0)
            if task.eid in done_tasks:
                continue
            done_tasks.append(task.eid)
            the_dates = []
            event = task.cw_adapt_to('ICalendarable')
            tstart = event.start
            tstop = event.stop
            if tstart:
                tstart = todate(tstart)
                if tstart != day:
                    continue
                the_dates = [tstart]
            if tstop:
                tstop = todate(tstop)
                if tstop != day:
                    continue
                the_dates = [tstop]
            if tstart and tstop:
                the_dates = date_range(max(tstart, day),
                                       min(tstop, day))
            if not the_dates:
                continue

            if task not in task_colors:
                task_colors[task] = colors[next_color_index]
                next_color_index = (next_color_index+1) % len(colors)

            # add task description for the given day
            task_descr = calendar._TaskEntry(task, task_colors[task])
            task_descrs.append(task_descr)

        self.w(u'<div id="oneweekcalid">')
        # build schedule for the given day
        self._build_schedule(day, task_descrs, title)

        self.w(u'</div>')
        self.w(u'<div id="coord"></div>')
        self.w(u'<div id="debug">&#160;</div>')

    def _build_schedule(self, date, task_descrs, title):
        _cw = self._cw
        if 'year' in _cw.form:
            year = int(_cw.form['year'])
        else:
            year = date.year
        if 'week' in _cw.form:
            week = int(_cw.form['week'])
        else:
            week = date.isocalendar()[1]
        self.w(u'<table class="omcalendar" id="week">')
        self.w(u'<tr><th class="transparent"></th></tr>')

        # output header
        self.w(u'<tr>')
        self.w(u'<th class="transparent"></th>') # column for hours
        _today = date.today()
        wdate = date
        if wdate.isocalendar() == _today.isocalendar():
            if title:
                self.w(u'<th class="today">%s<br/>%s %s</th>' %
                       (title, _cw._(calendar.WEEKDAYS[date.weekday()]),
                        _cw.format_date(wdate)))
            else:
                self.w(u'<th class="today">%s<br/>%s</th>' %
                       (_cw._(calendar.WEEKDAYS[date.weekday()]),
                        _cw.format_date(wdate)))
        else:
            if title:
                self.w(u'<th>%s<br/>%s %s</th>' %
                       (title, _cw._(calendar.WEEKDAYS[date.weekday()]),
                        _cw.format_date(wdate)))
            else:
                self.w(u'<th>%s<br/>%s</th>' %
                       (_cw._(calendar.WEEKDAYS[date.weekday()]),
                        _cw.format_date(wdate)))
        self.w(u'</tr>')

        # build day calendar
        self.w(u'<tr>')
        self.w(u'<td style="width:1%;">') # column for hours
        extra = u''
        for h in range(8, 20):
            self.w(u'<div class="hour" %s>'%extra)
            self.w(u'%02d:00'%h)
            self.w(u'</div>')
        self.w(u'</td>')

        wdate = date
        classes = ""
        if wdate.isocalendar() == _today.isocalendar():
            classes = " today"
        self.w(u'<td class="column %s" id="%s">' % (classes, calendar.WEEKDAYS[date.weekday()]))
        if len(self.cw_rset.column_types(0)) == 1:
            etype = list(self.cw_rset.column_types(0))[0]
            url = _cw.build_url(vid='creation', etype=etype,
                                     schedule=True,
                                     __redirectrql=self.cw_rset.printable_rql(),
                                     __redirectparams=_cw.build_url_params(year=year, week=week),
                                     __redirectvid=self.__regid__
                                 )
            extra = (u' ondblclick="addCalendarItem(event, hmin=8, hmax=20, '
                     u'year=%s, month=%s, day=%s, duration=2, baseurl=\'%s\')"'
                     % (wdate.year, wdate.month, wdate.day, xml_escape(url)))
        else:
            extra = u''
        self.w(u'<div class="columndiv"%s>'% extra)
        for h in range(8, 20):
            self.w(u'<div class="hourline" style="top:%sex;">'%((h-7)*8))
            self.w(u'</div>')
        self._build_calendar_cell(wdate, task_descrs)

        self.w(u'</div>')
        self.w(u'</td>')
        self.w(u'</tr>')
        self.w(u'</table>')

    def _build_calendar_cell(self, date, task_descrs):
        inday_tasks = [t for t in task_descrs
                       if t.is_one_day_task() and t.in_working_hours()]
        wholeday_tasks = [t for t in task_descrs if not t.is_one_day_task()]
        inday_tasks.sort(key=lambda t:t.task.cw_adapt_to('ICalendarable').start)
        sorted_tasks = []
        for i, t in enumerate(wholeday_tasks):
            t.index = i
        ncols = len(wholeday_tasks)
        while inday_tasks:
            t = inday_tasks.pop(0)
            for i, c in enumerate(sorted_tasks):
                if not c or c[-1].task.cw_adapt_to('ICalendarable').stop <= t.task.cw_adapt_to('ICalendarable').start:
                    c.append(t)
                    t.index = i+ncols
                    break
            else:
                t.index = len(sorted_tasks) + ncols
                sorted_tasks.append([t])
        ncols += len(sorted_tasks)
        if ncols == 0:
            return

        inday_tasks = []
        for tasklist in sorted_tasks:
            inday_tasks += tasklist
        width = 100.0/ncols
        for task_desc in wholeday_tasks + inday_tasks:
            task = task_desc.task
            start_hour = 8
            start_min = 0
            stop_hour = 20
            stop_min = 0
            event = task.cw_adapt_to('ICalendarable')
            if event.start:
                if todatetime(date) < todatetime(event.start) < todatetime(date + calendar.ONEDAY):
                    start_hour = max(8, event.start.hour)
                    start_min = event.start.minute
            if event.stop:
                if todatetime(date) < todatetime(event.stop) < todatetime(date + calendar.ONEDAY):
                    stop_hour = min(20, event.stop.hour)
                    if stop_hour < 20:
                        stop_min = event.stop.minute

            height = 100.0*(stop_hour+stop_min/60.0-start_hour-start_min/60.0)/(20-8)
            top = 100.0*(start_hour+start_min/60.0-8)/(20-8)
            left = width*task_desc.index
            style = "height: %s%%; width: %s%%; top: %s%%; left: %s%%; " % \
                (height, width, top, left)
            self.w(u'<div class="task %s" style="%s">' % \
                       (task_desc.color, style))
            task.view('calendaritem', dates=False, w=self.w)
            params = self._cw.build_url_params(year=date.year, week=date.isocalendar()[1])
            url = task.absolute_url(vid='edition',
                                    __redirectrql=self.cw_rset.printable_rql(),
                                    __redirectparams=params,
                                    __redirectvid=self.__regid__
                                    )

            self.w(u'<div class="tooltip" ondblclick="stopPropagation(event); '
                   u'window.location.assign(\'%s\'); return false;">'
                   % xml_escape(url))
            task.view('tooltip', w=self.w)
            self.w(u'</div>')
            if event.start is None:
                self.w(u'<div class="bottommarker">'
                       u'<div class="bottommarkerline" '
                       u'style="margin: 0px 3px 0px 3px; height: 1px;"></div>'
                       u'<div class="bottommarkerline" '
                       u'style="margin: 0px 2px 0px 2px; height: 1px;"></div>'
                       u'<div class="bottommarkerline" '
                       u'style="margin: 0px 1px 0px 1px; height: 3ex; '
                       u'color: white; font-size: x-small; vertical-align: center; '
                       u'text-align: center;">end</div>'
                       u'</div>')
            self.w(u'</div>')




### Redefinition of the workflows permissions

# see postcreate.py
CONF_CHAIR = 'U is_chair_at X'
TALK_CHAIR = 'U is_chair_at C, X in_conf C, X is Talk'
REVIEWER = 'U is_reviewer_at C, X in_conf C'
OWNER_CALL_OPEN = 'X owned_by U, X in_conf C, C call_open True'
MANAGER = ('managers',)
OWNER_MANAGER = ('owners', 'managers')

def set_workflow_permissions(perms):
    for name, permissions in perms.items():
        requiredgroups, conditions = permissions
        tr = rql('Transition T WHERE T name %(name)s', {'name': name})
        assert len(tr)==1
        tr = tr.entities().next()
        tr.set_permissions(requiredgroups, conditions, reset=True)

# Conference Workflow
redefine_conference_permissions = {
    'schedule':           (MANAGER,       CONF_CHAIR),
    'archive':            (MANAGER,       CONF_CHAIR),
}

# Talk Workflow
redefine_talk_permissions = {
    'submit your work':   (MANAGER,       (TALK_CHAIR, OWNER_CALL_OPEN) ),
    'redraft':            (MANAGER,       (TALK_CHAIR, OWNER_CALL_OPEN) ),
    'send to reviewer':   (MANAGER,       (TALK_CHAIR,)                 ),
    'need correction':    (MANAGER,       (REVIEWER, TALK_CHAIR)        ),
    'resend to reviewer': (OWNER_MANAGER, (TALK_CHAIR,)                 ),
    'propose to accept':  (MANAGER,       (REVIEWER, TALK_CHAIR)        ),
    'propose to reject':  (MANAGER,       (REVIEWER, TALK_CHAIR)        ),
    'need more review':   (MANAGER,       (REVIEWER, TALK_CHAIR)        ),
    'accept talk':        (MANAGER,       (TALK_CHAIR,)                 ),
    'reject talk':        (MANAGER,       (TALK_CHAIR,)                 ),
}

set_workflow_permissions(redefine_conference_permissions)
set_workflow_permissions(redefine_talk_permissions)
commit()



# postcreate script. You could setup a workflow here for example

_ = unicode

# constants used for permissions

TALK_CHAIR = 'U is_chair_at C, X in_conf C, X is Talk'
CONF_CHAIR = 'U is_chair_at X'
REVIEWER = 'U is_reviewer_at C, X in_conf C'
OWNER_CALL_OPEN = 'X owned_by U, X in_conf C, C call_open True'
MANAGER = ('managers',)
OWNER_MANAGER = ('owners', 'managers')

# Conference Workflow

def define_conference_workflow():
    twf = add_workflow(_('conference workflow'), 'Conference')
    planned = twf.add_state(_('planned'), initial=True)
    scheduled = twf.add_state(_('scheduled'))
    archived = twf.add_state(_('archived'))
    twf.add_transition(_('schedule'), (planned,), scheduled, MANAGER, conditions=CONF_CHAIR)
    twf.add_transition(_('archive'), (scheduled,), archived, MANAGER, conditions=CONF_CHAIR)

define_conference_workflow()

# Talk Workflow

def define_talk_workflow():
    twf = add_workflow(_('talk workflow'), 'Talk')
    draft = twf.add_state(_('draft'), initial=True)
    submitted = twf.add_state(_('submitted'))
    correction = twf.add_state(_('correction'))
    inreview = twf.add_state(_('inreview'))
    accept_pending = twf.add_state(_('accept_pending'))
    reject_pending = twf.add_state(_('reject_pending'))
    accepted = twf.add_state(_('accepted'))
    rejected = twf.add_state(_('rejected'))
    twf.add_transition(_('submit your work'), (draft,), submitted, MANAGER, conditions=(TALK_CHAIR,OWNER_CALL_OPEN))
    twf.add_transition(_('redraft'), (submitted,), draft, MANAGER, conditions=(TALK_CHAIR,OWNER_CALL_OPEN))
    twf.add_transition(_('send to reviewer'), (submitted,), inreview, MANAGER, conditions=TALK_CHAIR)
    twf.add_transition(_('need correction'), (inreview,), correction, MANAGER, conditions=(REVIEWER,TALK_CHAIR))
    twf.add_transition(_('resend to reviewer'), (correction,), inreview, OWNER_MANAGER, conditions=TALK_CHAIR)
    twf.add_transition(_('propose to accept'), (inreview,), accept_pending, MANAGER, conditions=(REVIEWER,TALK_CHAIR))
    twf.add_transition(_('propose to reject'), (inreview,), reject_pending, MANAGER, conditions=(REVIEWER,TALK_CHAIR))
    twf.add_transition(_('need more review'), (accept_pending,reject_pending), inreview, MANAGER, conditions=(REVIEWER,TALK_CHAIR))
    twf.add_transition(_('accept talk'), (accept_pending,reject_pending), accepted, MANAGER, conditions=TALK_CHAIR)
    twf.add_transition(_('reject talk'), (reject_pending,accept_pending), rejected, MANAGER, conditions=TALK_CHAIR)

define_talk_workflow()

commit()



# rename a transition, then allow managers to pass two transitions
rql('SET T name "resend to reviewer" WHERE T is Transition, T name "submit your work", '
    'T transition_of W, W workflow_of X, X name "Talk", S allowed_transition T, S name "correction"')
rset = rql('Any T WHERE T is Transition, T name "submit your work", '
           'T transition_of W, W workflow_of X, X name "Talk"')
transition = rset.get_entity(0,0)
transition.set_permissions(requiredgroups=('managers',), reset=False)
rset = rql('Any T WHERE T is Transition, T name "resend to reviewer", '
           'T transition_of W, W workflow_of X, X name "Talk"')
transition = rset.get_entity(0,0)
transition.set_permissions(requiredgroups=('managers',), reset=False)

commit()

# change cardinality of relation 'leads' and maxsize of Talk.title
sync_schema_props_perms()




from __future__ import with_statement
from logilab.mtconverter import xml_escape

from cubicweb.predicates import is_instance, has_related_entities
from cubicweb.predicates import authenticated_user
from cubicweb.web.views.idownloadable import DownloadBox
from cubicweb.web.component import EntityCtxComponent
from cubicweb.web.views import primary, calendar
from cubicweb.web.views.ajaxcontroller import ajaxfunc

_ = unicode

class TalkPrimaryView(primary.PrimaryView):
    __select__ = is_instance('Talk',)

    def render_entity_title(self, entity):
        self.w(u'<h1><span class="etype">%s</span> %s</h1>'
               % (entity.dc_type().capitalize(), xml_escape(entity.dc_title())))
        self.w(u'<table class="talk"><tr><td>')
        self.render_talk_info(entity)
        self.w(u'</td><td>')
        self.w(u'</td></tr></table>')

    def render_talk_info(self, entity):
        self.w(u'<div>')
        self.w(u'<span class="speaker">%s %s</span>' % (
                self._cw._(u'Presented by'),
                self._cw.view('csv', entity.related('leads', 'object'),
                              subvid='outofcontext')))
        if entity.track:
            self.w(u' %s <span>' % self._cw._(u'in'))
            self.w(u'%s </span> ' % entity.track.view('incontext'))
        self.render_talk_time_place(entity)
        self.w(u'</div>')

    def render_talk_time_place(self, entity):
        _ = self._cw._
        if entity.start_time or entity.end_time:
            self.w(u'<span class="talktime">')
            if entity.start_time and entity.end_time:
                self.w(_(u"on %(sdate)s from %(stime)s to %(etime)s") %
                       ({'sdate': self._cw.format_date(entity.start_time),
                         'stime': self._cw.format_time(entity.start_time),
                         'etime': self._cw.format_time(entity.end_time)}))
            self.w(u'</span>')
        if entity.location:
            self.w(u'<span> %s %s</span>' % (_(u'in room'), xml_escape(entity.location)))

    def render_entity_attributes(self, entity):
        if entity.description:
            self.w(u'<h5>%s</h5>' % self._cw._(u'Abstract'))
            self.w(u'<div>%s</div>' % entity.dc_description('text/html'))

class AttendanceComponent(EntityCtxComponent):
    __regid__ = 'attendance'
    __select__ = (EntityCtxComponent.__select__ & is_instance('Talk') &
                  authenticated_user())
    context = 'ctxtoolbar'

    def init_rendering(self):
        _ = unicode
        self._cw.add_js('cubes.conference.js')
        user = self._cw.user
        eid = self.entity.eid
        rset = self._cw.execute('Any X WHERE U attend X, U eid %(u)s, X eid %(x)s',
                                {'u': user.eid, 'x': eid})
        if rset.rowcount:
            # user is already attending this talk
            self.klass = 'delete_attend'
            self.title = _('click here if you do not plan to attend this talk')
            self.label, self.label_id = _('You plan to attend'), 'attend'
        else:
            # user is not attending the talk
            self.klass = 'set_attend'
            self.title = _('click here if you plan to attend this talk')
            self.label, self.label_id = _('You do not plan to attend'), 'unattend'
        # count attendees
        rql = 'Any COUNT(U) WHERE U attend X, X eid %s' % eid
        self.nb_person = self._cw.execute(rql)[0][0]

    def render_body(self, w):
        w(u'<div class="%s" id="%s" data-eid="%s">' % (self.__regid__, self.domid, self.entity.eid))
        w(u'<div class="attendbutton %s" title="%s" id="%s">'
               u'<div id="attendinfo">%s</div><div>(%s %s)</div></div>'
               % (self.klass, self._cw._(self.title),
                  self.label_id, self._cw._(self.label), self.nb_person,
                  self._cw._('people plan to attend')))
        w(u'</div>')


@ajaxfunc
def conference_delete_attend(self, talkeid):
    usereid = self._cw.user.eid
    self._cw.execute('DELETE U attend X WHERE U eid %(u)s, X eid %(x)s', {'u': usereid, 'x': talkeid})

@ajaxfunc
def conference_set_attend(self, talkeid):
    usereid = self._cw.user.eid
    self._cw.execute('SET U attend X WHERE U eid %(u)s, X eid %(x)s', {'u': usereid, 'x': talkeid})


class TalkCalendarItem(calendar.CalendarItemView):
    __select__ = is_instance('Talk')

    def cell_call(self, row, col, dates=False):
        talk = self.cw_rset.complete_entity(row)
        persons = self._cw.execute('Any P WHERE P leads X, X eid %(x)s', {'x': talk.eid}).entities()
        self.w(u'<a href="%s">%s - %s</a>' % (talk.absolute_url(), xml_escape(talk.dc_title()),
                                              xml_escape(', '.join(p.dc_long_title() for p in persons))))
        if talk.location:
            self.w(u'<div>[%s %s]</div>' % (self._cw._(u'room'), talk.location))


class AttachmentsDownloadBox(DownloadBox):
    """A box containing all downloadable attachments of a Talk"""
    __regid__ = 'attachments_downloads_box'
    __select__ = is_instance('Talk') & has_related_entities('has_attachments', 'subject')
    title = _('Attachments')

    def init_rendering(self):
        self.items = self.entity.has_attachments



from logilab.common.date import date_range, ONEDAY
from logilab.mtconverter import xml_escape

from cubicweb.predicates import is_instance, rql_condition
from cubicweb.view import EntityView
from cubicweb.web.views import tabs

_ = unicode

class TrackPrimaryView(tabs.TabbedPrimaryView):
    __select__ = is_instance('Track')
    tabs = [_('trackinfo'), _('tracktalkslist'), _('tracktalksschedule')]
    default_tab = 'trackinfo'

    def render_entity_title(self, entity):
        self.w(u'<h1>%s</h1>' % xml_escape(entity.dc_long_title()))

class TrackInfoView(EntityView):
    __select__ = is_instance('Track')
    __regid__ = 'trackinfo'

    def cell_call(self, row, col):
        entity = self.cw_rset.get_entity(row, col)
        if entity.description:
            self.w(u'<div>%s</div>' % entity.dc_description('text/html'))

class TrackTalksList(EntityView):
    __select__ = is_instance('Track')
    __regid__ = 'tracktalkslist'

    def cell_call(self, row, col):
        entity = self.cw_rset.get_entity(row, col)
        _ = self._cw._
        rql = ('Any T, group_concat(U), ST, SP, L GROUPBY T, ST, SP, L '
               'ORDERBY ST WHERE T location L, T start_time ST, '
               'T end_time SP, U leads T, T in_track TR, T in_state S, '
               'S name "accepted", TR eid %(x)s')
        rset = self._cw.execute(rql, {'x': entity.eid})
        if rset:
            headers = (_(u'Talk title'), _(u'Speaker'), _(u'Begin'), _(u'End'), _(u'Location'))
            self.wview('table', rset, headers=headers, cellvids={1:'users_list'})
        else:
            self.w(u'<div>%s</div>' % _(u'No accepted talk yet in this track'))

class TrackTalksSchedule(EntityView):
    __select__ = is_instance('Track') & rql_condition('T in_track X')
    __regid__ = 'tracktalksschedule'

    def cell_call(self, row, col):
        track = self.cw_rset.get_entity(row, col)
        talks = self._cw.execute('Any T WHERE T in_track TR, TR eid %(e)s, '
                                 'T in_state S, S name "accepted"',
                                 {'e': track.eid})
        if not talks:
            return
        track_cal = track.cw_adapt_to('ICalendarable')
        for day in date_range(track_cal.start, track_cal.stop+ONEDAY):
            self.w(u'<table style="width:100%">')
            self.w(u'<tr>')
            self.w(u'<td>')
            self.wview('onedaycal', talks, initargs={'day': day, 'title': track.title})
            self.w(u'</td>')
            self.w(u'</tr></table>')



add_cube('forgotpwd')

# add_entity_type('Sponsor', auto=True) not necessary, add_cube above is greedy

set_property('actions.about.visible', u'')
set_property('actions.changelog.visible', u'')
set_property('boxes.blog_summary_box.visible', u'')

# change talk workflow

talk_states = rql('Any T, N ORDERBY T WHERE T in_state S, S name N, T is Talk').rows

rql('DELETE Workflow WF WHERE WF workflow_of X, X name "Talk"')

def define_talk_workflow():
    twf = add_workflow(_('talk workflow'), 'Talk')
    draft = twf.add_state(_('draft'), initial=True)
    submitted = twf.add_state(_('submitted'))
    correction = twf.add_state(_('correction'))
    inreview = twf.add_state(_('inreview'))
    accepted = twf.add_state(_('accepted'))
    rejected = twf.add_state(_('rejected'))
    twf.add_transition(_('submit your work'), (draft,), submitted,
                       ('owners',))
    twf.add_transition(_('send to reviewer'), (submitted,), inreview,
                       ('managers',))
    twf.add_transition(_('ask for correction'), (inreview,), correction,
                       ('managers',), conditions=('U reviews X',))
    twf.add_transition(_('submit your work'), (correction,), inreview,
                       ('owners',))
    twf.add_transition(_('accept talk'), (inreview,), accepted,
                       ('managers',), conditions=('U reviews X',))
    twf.add_transition(_('reject talk'), (inreview,), rejected,
                       ('managers',), conditions=('U reviews X',))

define_talk_workflow()

urql = session.unsafe_execute

for ent, name in talk_states:
    if name not in ('accepted', 'rejected'):
        name = _('draft')
    urql('SET X in_state S WHERE X eid %(x)s, S name %(name)s, X is ET, '
         'ET default_workflow WF, S state_of WF, WF workflow_of T, '
         'T name "Talk"', {'x': ent, 'name': name}, 'x')

commit()

sync_schema_props_perms()

commit()
